from imucal.calibration_info import CalibrationInfo  # noqa: F401
from imucal.ferraris_calibration import (
    FerrarisCalibration,
    TurntableCalibration,
    ferraris_regions_from_df,
    ferraris_regions_from_interactive_plot,
    ferraris_regions_from_section_list,
    FerrarisSignalRegions,
)
from imucal.ferraris_calibration_info import FerrarisCalibrationInfo, TurntableCalibrationInfo  # noqa: F401

__all__ = [
    "CalibrationInfo",
    "FerrarisCalibration",
    "TurntableCalibration",
    "FerrarisSignalRegions",
    "FerrarisCalibrationInfo",
    "TurntableCalibrationInfo",
    "ferraris_regions_from_df",
    "ferraris_regions_from_interactive_plot",
    "ferraris_regions_from_section_list",
]



"""Helper providing a small GUI to label timeseries data."""
import tkinter as tk
from collections import OrderedDict
from itertools import chain
from tkinter.messagebox import showinfo
from typing import Sequence, Optional

import numpy as np
import pandas as pd


class CalibrationGui:
    """A Gui that can be used to label the different required sections of a calibration.

    For details see the `manual` string.

    Examples
    --------
    >>> # This will launch the GUI and block execution until closed
    >>> gui = CalibrationGui(acc, gyro, ['label1', 'label2'])
    >>> # While the GUI is open the sections are labeled. Once closed the next line is executed
    >>> labels = gui.section_list
    >>> labels['label1']  # First value is start, second value is end of region
    (12, 355)
    >>> labels['label2']  # First value is start, second value is end of region
    (500, 758)

    """

    section_list = None
    acc_list_markers = None
    gyro_list_markers = None
    expected_labels = None

    manual = """
    Mark the start and end point of each section listed in the sidebar in the plots.
    The section that is currently labeled is marked in blue in the sidebar.
    You can use either <return> (<Enter>) to advance to the next section, which has missing labels
    or click a section label manual.

    To create a mark click either with the left or the right mouse button on the plot.
    For each section you can place one RightClick and one LeftClick label.
    It does not matter, which you place first.
    If both labels are placed, the region in between them is colored.
    Now you can either press Enter (or click on any other label in the sidebar) to continue with labeling the next
    section or you can adjust the labels by repeated left and right clicks until you're satisfied.
    """

    def __init__(
        self,
        acc: np.ndarray,
        gyro: np.ndarray,
        expected_labels: Sequence[str],
        title: Optional[str] = None,
        master: Optional[tk.Frame] = None,
    ):
        """Launch new GUI instance.

        Parameters
        ----------
        acc :
            3D array containing all acceleration data
        gyro :
            3D array containing all gyroscope data
        expected_labels :
            List of all label names that should be labeled
        title :
            Title displayed in the titlebar of the GUI
        master :
            Parent window if GUI should be embedded in larger application

        """
        import matplotlib  # noqa: import-outside-toplevel
        from matplotlib.backends.backend_tkagg import (  # noqa: import-outside-toplevel
            FigureCanvasTkAgg,
            NavigationToolbar2Tk,
        )

        self.expected_labels = expected_labels
        cmap = matplotlib.cm.get_cmap("Set3")
        self.colors = {k: matplotlib.colors.to_hex(cmap(i / 12)) for i, k in enumerate(expected_labels)}

        self.text_label = "Labels set: {{}}/{}".format(len(expected_labels))

        if not master:
            master = tk.Tk()

        master.title(title or "Calibration Gui")
        master.bind("<Return>", lambda x: self._select_next(self.labels.curselection()[0]))

        # reset variables
        self.section_list = OrderedDict(((k, [None, None]) for k in expected_labels))
        self.acc_list_markers = {k: [] for k in expected_labels}
        self.gyro_list_markers = {k: [] for k in expected_labels}

        self.main_area = tk.Frame(master=master, relief=tk.RAISED)
        self.main_area.pack(fill=tk.BOTH, expand=1)

        self.side_bar = tk.Frame(master=self.main_area)
        self.side_bar.pack(fill=tk.Y, side=tk.RIGHT)
        self._create_sidebar()

        # Create a container
        self.fig, self.axs = _create_figure(acc, gyro)

        self.canvas = FigureCanvasTkAgg(self.fig, master=self.main_area)
        self.canvas.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=1)

        self.canvas.mpl_connect("button_press_event", self._onclick)

        self.label_text = tk.Text(self.main_area, height=1, width=80)
        self.label_text.pack(side=tk.TOP, fill=tk.X, expand=0)
        self.label_text.insert(tk.END, self.text_label.format(str(0)))
        toolbar = NavigationToolbar2Tk(self.canvas, self.main_area)
        toolbar.update()
        toolbar.pack(side=tk.TOP, fill=tk.X, expand=0)

        self.labels.selection_anchor(0)
        self.labels.selection_set(0)

        master.mainloop()

    def _create_sidebar(self):
        self.labels = tk.Listbox(master=self.side_bar, width=30)
        self.labels.pack(side=tk.TOP, fill=tk.BOTH, expand=1)
        help_button = tk.Button(master=self.side_bar, height=2, text="Help", command=self._show_help)
        help_button.pack(side=tk.BOTTOM, fill=tk.BOTH)

        for key in self.section_list:
            self.labels.insert(tk.END, key)

        self._update_list_box()

    def _show_help(self):
        showinfo("Window", self.manual)

    def _select_next(self, current):
        next_val = (current + 1) % self.labels.size()

        if all(list(self.section_list.values())[next_val]):
            if self._n_labels() == len(self.section_list):
                return

            self._select_next(next_val)

        self.labels.selection_clear(0, tk.END)
        self.labels.selection_anchor(next_val)
        self.labels.selection_set(next_val)

    def _update_list_box(self):
        for i, (key, val) in enumerate(self.section_list.items()):
            self.labels.itemconfig(i, {"bg": self.colors[key]})
            if all(val):
                self.labels.itemconfig(i, {"fg": "#a5a9af"})

    def _onclick(self, event):
        # Only listen to left and right mouse clicks and only if we are not in zoom or drag mode.
        if event.button not in [1, 3] or str(self.canvas.toolbar.mode):
            return

        selected_key = self.labels.get(self.labels.curselection())

        if event.button == 1:
            x = int(event.xdata)
            self.section_list[selected_key][0] = x
        elif event.button == 3:
            x = int(event.xdata)
            self.section_list[selected_key][1] = x

        if all(self.section_list[selected_key]):
            self.section_list[selected_key].sort()

        self._update_marker(selected_key)

        self.label_text.delete("1.0", tk.END)
        self.label_text.insert(tk.END, self.text_label.format(str(self._n_labels())))

        self._update_list_box()

    def _update_marker(self, key):

        for line in chain(self.acc_list_markers[key], self.gyro_list_markers[key]):
            line.remove()
            self.acc_list_markers[key] = []
            self.gyro_list_markers[key] = []

        for val in self.section_list[key]:
            if val:
                marker_acc = self.axs[0].axvline(val, c=self.colors[key])
                marker_gyro = self.axs[1].axvline(val, c=self.colors[key])
                self.acc_list_markers[key].append(marker_acc)
                self.gyro_list_markers[key].append(marker_gyro)

        if all(self.section_list[key]):
            a_acc = self.axs[0].axvspan(
                self.section_list[key][0], self.section_list[key][1], alpha=0.5, color=self.colors[key]
            )
            a_gyr = self.axs[1].axvspan(
                self.section_list[key][0], self.section_list[key][1], alpha=0.5, color=self.colors[key]
            )
            self.acc_list_markers[key].append(a_acc)
            self.gyro_list_markers[key].append(a_gyr)

        self.fig.canvas.draw()
        self.fig.canvas.flush_events()

    def _n_labels(self):
        return sum((all(v) for v in self.section_list.values()))


def _convert_data_from_section_list_to_df(data: pd.DataFrame, section_list: pd.DataFrame) -> pd.DataFrame:
    out = dict()

    for label, row in section_list.iterrows():
        out[label] = data.iloc[row.start : row.end]

    return pd.concat(out)


def _create_figure(acc, gyro):
    from matplotlib.figure import Figure  # noqa: import-outside-toplevel

    fig = Figure(figsize=(20, 10))
    ax1 = fig.add_subplot(211)
    lines = ax1.plot(acc)
    ax1.grid(True)
    ax1.set_title("Use this plot to find the static regions for acc calibration")
    ax1.set_xlabel("time [s]")
    ax1.set_ylabel("acceleration [m/s^2]")
    ax1.legend(lines, list("xyz"))
    ax2 = fig.add_subplot(212, sharex=ax1)
    lines = ax2.plot(gyro)
    ax2.grid(True)
    ax2.set_title("Use this plot to find the single axis rotations for gyro calibration")
    ax2.set_xlabel("time [s]")
    ax2.set_ylabel("angular velocity [°/s]")
    ax2.legend(lines, list("xyz"))
    fig.tight_layout()
    return fig, (ax1, ax2)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['imucal']

package_data = \
{'': ['*']}

install_requires = \
['numpy>=1.19.2,<2.0.0',
 'pandas>=1.1.3,<2.0.0',
 'typing-extensions>=3.7.4,<4.0.0']

extras_require = \
{'calplot': ['matplotlib>=3.3.2,<4.0.0'], 'h5py': ['h5py>=2.10.0,<3.0.0']}

setup_kwargs = {
    'name': 'imucal',
    'version': '2.0.1',
    'description': 'A Python library to calibrate 6 DOF IMUs',
    'long_description': '# IMU Calibration\n![Test and Lint](https://github.com/mad-lab-fau/imucal/workflows/Test%20and%20Lint/badge.svg)\n[![Documentation Status](https://readthedocs.org/projects/imucal/badge/?version=latest)](https://imucal.readthedocs.io/en/latest/?badge=latest)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![PyPI](https://img.shields.io/pypi/v/imucal)](https://pypi.org/project/imucal/)\n![PyPI - Downloads](https://img.shields.io/pypi/dm/imucal)\n\nThis package provides methods to calculate and apply calibrations for IMUs based on multiple different methods.\n\nSo far supported are:\n\n- Ferraris Calibration (Ferraris1995)\n- Ferraris Calibration using a Turntable\n\n## WARNING: VERSION UPDATE\n\nVersion 2.0 was recently released and contains multiple API breaking changes!\nTo learn more about that, check `Changelog.md`.\n\nIf you want to ensure that your old code still works, specify a correct version during install and in your\n`requirement.txt` files\n\n```\npip install "imucal<2.0"\n```\n\n## Installation\n\n```\npip install imucal\n```\n\nTo use the included calibration GUI you also need matplotlib (version >2.2).\nYou can install it using:\n\n```\npip install imucal[calplot]\n```\n\n## Quickstart\nThis package implements the IMU-infield calibration based on Ferraris1995.\nThis calibration methods requires the IMU data from 6 static positions (3 axis parallel and antiparallel to gravitation\nvector) and 3 rotations around the 3 main axis.\nIn this implementation these parts are referred to as follows `{acc,gry}_{x,y,z}_{p,a}` for the static regions and\n`{acc,gry}_{x,y,z}_rot` for the rotations.\nAs example, `acc_y_a` would be the 3D-acceleration data measured during a static phase, where the **y-axis** was \noriented **antiparallel** to the gravitation.\n\nTo annotate a Ferraris calibration session that was recorded in a single go, you can use the following code snippet.\nNote: This will open an interactive Tkinter plot.\nTherefore, this will only work on your local PC and not on a server or remote hosted Jupyter instance.\n\n```python\nfrom imucal import ferraris_regions_from_interactive_plot\n\n# Your data as a 6 column dataframe\ndata = ...\n\nsection_data, section_list = ferraris_regions_from_interactive_plot(\n    data, acc_cols=["acc_x", "acc_y", "acc_z"], gyr_cols=["gyr_x", "gyr_y", "gyr_z"]\n)\n# Save the section list as reference for the future\nsection_list.to_csv(\'./calibration_sections.csv\')  # This is optional, but recommended\n```\n\nNow you can perform the calibration\n```python\nfrom imucal import FerrarisCalibration\n\nsampling_rate = 100 #Hz \ncal = FerrarisCalibration()\ncal_mat = cal.compute(section_data, sampling_rate, from_acc_unit="m/s^2", from_gyr_unit="g")\n# `cal_mat` is you final calibration matrix object, you can use to calibrate data\ncal_mat.to_json_file(\'./calibration.json\')\n```\n\nApplying a calibration:\n\n```python\nfrom imucal.management import load_calibration_info\n\ncal_mat = load_calibration_info(\'./calibration.json\')\nnew_data = pd.DataFrame(...)\ncalibrated_data = cal_mat.calibrate_df(new_data, acc_unit="m/s^2", gyr_unit="deg/s")\n```\n\nFor further information on how to perform a calibration check the \n[User Guides](https://imucal.readthedocs.io/en/latest/guides/index.html) or the\n[examples](https://imucal.readthedocs.io/en/latest/auto_examples/index.html)\n\n## Further Calibration Methods\n\nAt the moment, this package only implements calibration methods based on Ferraris1994, because this is what we use to\ncalibrate our IMUs.\nWe are aware that various other methods exist and would love to add them to this package as well.\nUnfortunately, at the moment we can not justify the time requirement.\n\nStill, we think that this package provides a suitable framework to implement other calibration emthods with relative\neasy.\nIf you would like to contribute such a method, let us know on the github-issue page and we will try to help you as good\nas possible.\n\n## Contributing\n\nAll project management and development happens through this Github project.\nIf you have any issues, ideas, or any comments at all, just open a new issue.\nPlease be polite and considerate of our time.\nWe appreciate everyone who is using our software or even wants to improve it, but sometime other things come in the way,\nand it takes us a couple of days to get back to you.\n',
    'author': 'Arne Küderle',
    'author_email': 'arne.kuederle@fau.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/mad-lab-fau/imucal',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
